@@ -121,6 +121,7 @@
 \\stopTextSpan
 \\stopStaff
 \\stopGroup
+\\stopGraceSlur
 \\stopGraceMusic
 \\stopAppoggiaturaMusic
 \\stopAcciaccaturaMusic
@@ -133,6 +134,7 @@
 \\startTextSpan
 \\startStaff
 \\startGroup
+\\startGraceSlur
 \\startGraceMusic
 \\startAppoggiaturaMusic
 \\startAcciaccaturaMusic
